@@ -25,7 +25,7 @@ use strict;
 use vars qw( $VERSION );
 
 BEGIN {
-    ($VERSION) = '$Revision: 1.3 $' =~ /Revision: ([\d.]+)/;
+    ($VERSION) = '$Revision: 1.7 $' =~ /Revision: ([\d.]+)/;
 }
 
 =pod
@@ -319,12 +319,13 @@ sub validate {
 
     # Apply specific filters
     while ( my ($field,$filters) = each %{$profile->{field_filters} }) {
-	foreach my $filter ( @{$profile->{filters}} ) {
+	my @f = ref $filters eq "ARRAY" ? @$filters : ( $filters );
+	foreach my $filter ( @f ) {
 	    # Qualify symbolic references
 	    $filter = ref $filter ? $filter : "filter_" . $filter;
 	    no strict 'refs';
 
-	    $valid{$field} = $filter->( $valid{$filter} );
+	    $valid{$field} = $filter->( $valid{$field} );
 	}
     }
 
@@ -430,7 +431,7 @@ sub filter_trim {
     # Remove whitespace at the end
     $value =~ s/\s+$//g;
 
-    $value;
+    return $value;
 }
 
 =pod
@@ -445,7 +446,9 @@ sub filter_strip {
     my $value = shift;
 
     # Strip whitespace
-    $value =~ s/\s+/ /g
+    $value =~ s/\s+/ /g;
+
+    return $value;
 }
 
 =pod
@@ -459,7 +462,8 @@ Remove non digits characters from the input.
 sub filter_digit {
     my $value = shift;
     $value =~ s/\D//g;
-    $value;
+
+    return $value;
 }
 
 =pod
@@ -473,9 +477,200 @@ Remove non alphanumerical characters from the input.
 sub filter_alphanum {
     my $value = shift;
     $value =~ s/\W//g;
-    $value;
+    return $value;
 }
 
+=pod
+
+=item integer
+
+Extract from its input a valid integer number.
+
+=cut
+
+sub filter_integer {
+    my $value = shift;
+    $value =~ tr/0-9+-//dc;
+    ($value) =~ m/([-+]?\d+)/;
+    return $value;
+}
+
+=pod
+
+=item pos_integer
+
+Extract from its input a valid positive integer number.
+
+=cut
+
+sub filter_pos_integer {
+    my $value = shift;
+    $value =~ tr/0-9+//dc;
+    ($value) =~ m/(\+?\d+)/;
+    return $value;
+}
+
+=pod
+
+=item pos_integer
+
+Extract from its input a valid negative integer number.
+
+=cut
+
+sub filter_neg_integer {
+    my $value = shift;
+    $value =~ tr/0-9-//dc;
+    ($value) =~ m/(-\d+)/;
+    return $value;
+}
+
+=pod
+
+=item decimal
+
+Extract from its input a valid decimal number.
+
+=cut
+
+sub filter_decimal {
+    my $value = shift;
+    # This is a localization problem, but anyhow...
+    $value =~ tr/,/./;
+    $value =~ tr/0-9.+-//dc;
+    ($value) =~ m/([-+]?\d+\.?\d*)/;
+    return $value;
+}
+
+=pod
+
+=item pos_decimal
+
+Extract from its input a valid positive decimal number.
+
+=cut
+
+sub filter_pos_decimal {
+    my $value = shift;
+    # This is a localization problem, but anyhow...
+    $value =~ tr/,/./;
+    $value =~ tr/0-9.+//dc;
+    ($value) =~ m/(\+?\d+\.?\d*)/;
+    return $value;
+}
+
+=pod
+
+=item neg_decimal
+
+Extract from its input a valid negative decimal number.
+
+=cut
+
+sub filter_neg_decimal {
+    my $value = shift;
+    # This is a localization problem, but anyhow...
+    $value =~ tr/,/./;
+    $value =~ tr/0-9.-//dc;
+    ($value) =~ m/(-\d+\.?\d*)/;
+    return $value;
+}
+
+=pod
+
+=item dollars
+
+Extract from its input a valid number to express dollars like currency.
+
+=cut
+
+sub filter_dollars {
+    my $value = shift;
+    $value =~ tr/,/./;
+    $value =~ tr/0-9.+-//dc;
+    ($value) =~ m/(\d+\.?\d?\d?)/;
+    return $value;
+}
+
+=pod
+
+=item phone
+
+Filters out characters which aren't valid for an phone number. (Only
+accept digits [0-9], space, comma, minus, parenthesis, period and pound [#].)
+
+=cut
+
+sub filter_phone {
+    my $value = shift;
+    $value =~ tr/0-9,().#- //dc;
+    return $value;
+}
+
+=pod
+
+=item sql_wildcard
+
+Transforms shell glob wildcard (*) to the SQL like wildcard (%).
+
+=cut
+
+sub filter_sql_wildcard {
+    my $value = shift;
+    $value =~ tr/*/%/;
+    return $value;
+}
+
+=pod
+
+=item quotemeta
+
+Calls the quotemeta (quote non alphanumeric character) builtin on its
+input.
+
+=cut
+
+sub filter_quotemeta {
+    quotemeta $_[0];
+}
+
+=pod
+
+=item lc
+
+Calls the lc (convert to lowercase) builtin on its input.
+
+=cut
+
+sub filter_lc {
+    lc $_[0];
+}
+
+=pod
+
+=item uc
+
+Calls the uc (convert to uppercase) builtin on its input.
+
+=cut
+
+sub filter_uc {
+    uc $_[0];
+}
+
+=pod
+
+=item ucfirst
+
+Calls the ucfirst (Uppercase first letter) builtin on its input.
+
+=cut
+
+sub filter_ucfirst {
+    ucfirst $_[0];
+}
+
+
 =pod
 
 =back
@@ -613,7 +808,8 @@ contains at least 6 digits.)
 
 sub valid_phone {
     my $val = shift;
-    return $val =~ /\d{3}.*\d{3}/;
+
+    return $val =~ tr/0-9// >= 6;
 }
 
 =pod
@@ -621,13 +817,13 @@ sub valid_phone {
 =item american_phone
 
 This constraints checks if the number is a possible North American style
-of phone number : (XXX) XXX-XXXX. 7 or 10 digits long.
+of phone number : (XXX) XXX-XXXX. It has to contains more than 7 digits.
 
 =cut
 
 sub valid_american_phone {
     my $val = shift;
-    return  $val =~ /\d{3}.*?\d{4}/;
+    return $val =~ tr/0-9// >= 7;
 }
 
 =pod
